import os
from pathlib import Path
from typing import List

from termcolor import cprint
from jinja2 import Template

from kubetwo.common import Common
from kubetwo.config import ArtifactSettings, Settings
from kubetwo.exception import *


class Terraform:

    def __init__(self, cluster_name: str):
        self.artifact = ArtifactSettings(cluster_name)

    def initialize(self):
        command = "terraform init"
        try:
            cprint(f"Initializing Terraform...")
            Common.run_command(command, cwd=str(self.artifact.RENDERED_TERRAFORM_DIR_PATH), stdout=False)
            cprint(f"Terraform successfully initialized.\n")
        except ProcessException as e:
            raise TerraformInitException(str(e))

    def show_plan(self, destroy_mode=False):
        command = "terraform plan -compact-warnings"
        try:
            cprint(f"Preparing Terraform plan...")
            if destroy_mode:
                command = f"{command} -destroy"
            Common.run_command(command, cwd=str(self.artifact.RENDERED_TERRAFORM_DIR_PATH))
        except ProcessException as e:
            raise TerraformPlanException(str(e))

    def apply(self):
        command = "terraform apply -auto-approve"
        try:
            cprint(f"Changing AWS resources...")
            Common.run_command(command, cwd=str(self.artifact.RENDERED_TERRAFORM_DIR_PATH), stdout=False)
            cprint(f"Completed to change AWS resources.\n")
        except ProcessException as e:
            raise TerraformApplyException(str(e))

    def destroy(self):
        command = "terraform destroy -auto-approve"
        try:
            cprint(f"Removing AWS resources...")
            Common.run_command(command, cwd=str(self.artifact.RENDERED_TERRAFORM_DIR_PATH), stdout=False)
            cprint(f"Completed to remove AWS resources.\n")
        except ProcessException as e:
            raise TerraformDestroyException(str(e))

    def check_tf_plan(self):
        cprint(f"AWS resources will change.", "yellow")
        cprint("Do you proceed?\n", "yellow")
        try:
            choice = input("Enter a value [y/N]: ")
            print()
            if choice.lower() in ['y', 'yes']:
                return
        except KeyboardInterrupt:
            print()

        raise CheckDeniedException("[ERROR] Suspended to change AWS resources.")
    
    def create_terraform_manifests(
        self,
        cluster_name: str,
        control_plane: int,
        worker_node: int,
        ami: str,
        instance_type: str,
        availability_zone: str,
        open_ports: List[int],
        **kwargs
    ):
        cprint("Creating Terraform manifests...")
        ssh_public_key_path = Common.get_ssh_public_key_path()
        params = {
            "cluster_name": cluster_name.replace('_', '-'),
            "control_plane_count": control_plane,
            "worker_node_count": worker_node,
            "ami": ami,
            "instance_type": instance_type,
            "availability_zone": availability_zone,
            "ssh_public_key_name": str(ssh_public_key_path.name),
            "ssh_public_key": str(ssh_public_key_path),
            "open_ports": open_ports
        }
        tf_template_paths = list(Settings.TERRAFORM_TEMPLATE_DIR_PATH.glob("**/*.tf.j2"))
        for path in tf_template_paths:
            with open(path, "r") as file:
                template = Template(file.read())

            rendered_content = template.render(params)
            rendered_file_name = str(path.relative_to(Settings.TERRAFORM_TEMPLATE_DIR_PATH)).replace('.j2', '')
            rendered_file_path = self.artifact.RENDERED_TERRAFORM_DIR_PATH / rendered_file_name

            if rendered_file_path.parent != self.artifact.RENDERED_TERRAFORM_DIR_PATH:
                os.makedirs(str(rendered_file_path.parent), exist_ok=True)

            with open(rendered_file_path, "w") as f:
                f.write(rendered_content)

        cprint("Finished to create Terraform manifests.\n")



import json
from json.decoder import JSONDecodeError
import os
from pathlib import Path
from typing import Any, List

from jinja2 import Template

from kubetwo.config import ArtifactSettings, Settings


class Validator:
    """Collection of validation method for pydantic BaseModel"""

    @classmethod
    def str_to_path(cls, str_path: str) -> Path:
        try:
            return Path(str_path).expanduser()
        except:
            raise ValueError("is invalid format.")

    @classmethod
    def file_exists(cls, file_path: Path) -> Path:
        if not os.path.isfile(file_path):
            raise ValueError("file doesn't exist")
        return file_path

    @classmethod
    def artifact_dir_exists(cls, _, values: Any) -> Any:
        if not values.get("cluster_name"):
            return values

        artifact_dir_path = ArtifactSettings(values["cluster_name"]).ARTIFACT_DIR_PATH
        if not os.path.isdir(str(artifact_dir_path)):
            raise ValueError(f"\"{artifact_dir_path.name}\" doesn't exist in current place.")
        return values

    @classmethod
    def tfstate_exists(cls, _, values: Any) -> Any:
        if not values.get("cluster_name"):
            return values
            
        tfstate_path = ArtifactSettings(values["cluster_name"]).RENDERED_TFSTATE_PATH
        if not os.path.isfile(str(tfstate_path)):
            relative_tfstate_path = tfstate_path.relative_to(Path.cwd())
            raise ValueError(f"\"{relative_tfstate_path}\" doesn't exist in current place.")
        return values

    @classmethod
    def input_json_exists(cls, _, values: Any) -> Any:
        if not values.get("cluster_name"):
            return values

        input_json_file_path = ArtifactSettings(values["cluster_name"]).INPUT_JSON_FILE_PATH
        if not os.path.isfile(input_json_file_path):
            relative_input_json_file_path = input_json_file_path.relative_to(Path.cwd())
            raise ValueError(f"\"{relative_input_json_file_path}\" doesn't exist in current place.")
        return values

    @classmethod
    def is_input_json_valid(cls, _, values: Any) -> Any:
        if not values.get("cluster_name"):
            return values

        input_json_file_path = ArtifactSettings(values["cluster_name"]).INPUT_JSON_FILE_PATH
        relative_input_json_file_path = input_json_file_path.relative_to(Path.cwd())

        try:
            with open(input_json_file_path, "r") as f:
                json.load(f)
        except JSONDecodeError:
            raise ValueError(f"\"{relative_input_json_file_path}\" is invalid JSON format.")

        return values

    @classmethod
    def is_credentials_registered(cls, _, values: Any) -> Any:
        error_message = ""
        not_defined_environment_valiables = []
        for environment_variable_name in ["AWS_ACCESS_KEY_ID", "AWS_SECRET_ACCESS_KEY", "AWS_DEFAULT_REGION"]:
            if not os.environ.get(environment_variable_name):
                not_defined_environment_valiables.append(environment_variable_name)

        if not_defined_environment_valiables:
            error_message = ", ".join(not_defined_environment_valiables)
            if len(not_defined_environment_valiables) == 1:
                error_message += " is not defined as an environment variable"
            else:
                error_message += " are not defined as an environment variables"

        if error_message:
            raise ValueError(error_message)
            
        return values

    @classmethod
    def is_ssh_keys_registered(cls, _, values: Any) -> Any:
        error_message = ""
        not_defined_environment_valiables = []
        for environment_variable_name in ["SSH_PUBLIC_KEY_PATH", "SSH_PRIVATE_KEY_PATH"]:
            if not os.environ.get(environment_variable_name):
                not_defined_environment_valiables.append(environment_variable_name)

        if not_defined_environment_valiables:
            error_message = ", ".join(not_defined_environment_valiables)
            if len(not_defined_environment_valiables) == 1:
                error_message += " is not defined as an environment variable"
            else:
                error_message += " are not defined as an environment variables"

        if error_message:
            raise ValueError(error_message)
            
        return values

    @classmethod
    def is_ssh_keys_exist(cls, _, values: Any) -> Any:
        error_message = ""
        not_exist_ssh_keys = []

        for environment_variable_name in ["SSH_PUBLIC_KEY_PATH", "SSH_PRIVATE_KEY_PATH"]:
            key_str_path = os.environ.get(environment_variable_name)                
            try:
                Path(key_str_path).expanduser()
            except:
                not_exist_ssh_keys.append(environment_variable_name)
                continue
            
            if not Path(key_str_path).expanduser().exists():
                not_exist_ssh_keys.append(environment_variable_name)

        if not_exist_ssh_keys:
            error_message = ", ".join(not_exist_ssh_keys)
            if len(not_exist_ssh_keys) == 1:
                error_message += " doesn't exist"
            else:
                error_message += " don't exist"
        
        if error_message:
            raise ValueError(error_message)
        
        return values


class ValidationFormatter:

    @classmethod
    def format(cls, results: List) -> str:
        with open(Settings.VALIDATION_RESULT_FILE_PATH, "r") as file:
            template = Template(file.read())
        
        results_for_template = {}
        for result in results:
            locs = [str(elem) for elem in result["loc"]]
            loc = " ".join(locs)
            if loc == "__root__":
                loc = "others"
            if results_for_template.get(loc):
                results_for_template[loc].append(result["msg"])
            else:
                results_for_template[loc] = [result["msg"]]
            
        params = {"results": results_for_template}
        
        formatted_result = template.render(params)
        return formatted_result



import json
import sys
from typing import List

import fire
from pydantic import ValidationError
from termcolor import cprint

from kubetwo.create import Create
from kubetwo.scale import ScaleIn, ScaleOut
from kubetwo.delete import Delete
from kubetwo.model import *
from kubetwo.config import Settings
from kubetwo.exception import *
from kubetwo.validator import ValidationFormatter


def main():
    fire.Fire(CLI)


class CLI:

    @classmethod
    def create(
        cls,
        cluster_name: str,
        ami: str=Settings.DEFAULT_AMI,
        instance_type: str=Settings.DEFAULT_INSTANCE_TYPE,
        availability_zone: str=None,
        control_plane: int=Settings.DEFAULT_CONTROL_PLANE_NUM,
        worker_node: int=Settings.DEFAULT_WORKER_NODE_NUM,
        open_ports: List[int] = Settings.DEFAULT_OPEN_PORTS,
        approve: bool = False,
    ):
        try:
            cli_create_input = CLICreateInput(
                cluster_name = cluster_name,
                ami = ami,
                instance_type = instance_type,
                availability_zone = availability_zone,
                control_plane = control_plane,
                worker_node = worker_node,
                open_ports = open_ports,
                approve = approve
            )
            create_input = CreateInput(**cli_create_input.dict())
            Create(create_input).run()

        except ValidationError as e:
            cprint("[ERROR] Input value is not correct.", "red")
            validation_result = json.loads(e.json())
            cprint(ValidationFormatter.format(validation_result), "red")
            sys.exit(1)
        except (CheckDeniedException, TerraformStateNotFound) as e:
            cprint(e, "red")
            sys.exit(1)
        except TerraformException as e:
            cprint(e, "red")
            cprint("[ERROR] Terraform execution failed.", "red")
            sys.exit(1)
        except AnsibleException as e:
            cprint(e, "red")
            cprint("[ERROR] Provisioning by Ansible failed.", "red")
            sys.exit(1)

    
    @classmethod
    def scale(
        cls,
        cluster_name: str,
        worker_node: int=None,
    ):
        try:
            cli_scale_input = CLIScaleInput(
                cluster_name = cluster_name,
                worker_node = worker_node
            )
            scale_input = ScaleInput(**cli_scale_input.dict_all())

            if cli_scale_input.scale_type() == "in":
                ScaleIn(scale_input).run()
            elif cli_scale_input.scale_type() == "out":
                ScaleOut(scale_input).run()

        except ValidationError as e:
            cprint("[ERROR] Input value is not correct.", "red")
            validation_result = json.loads(e.json())
            cprint(ValidationFormatter.format(validation_result), "red")
            sys.exit(1)
        except (CheckDeniedException, TerraformStateNotFound) as e:
            cprint(e, "red")
            sys.exit(1)
        except TerraformException as e:
            cprint(e, "red")
            cprint("[ERROR] Terraform execution failed.", "red")
            sys.exit(1)
        except AnsibleException as e:
            cprint(e, "red")
            cprint("[ERROR] Provisioning by Ansible failed.", "red")
            sys.exit(1)
            
    @classmethod
    def delete(
        cls,
        cluster_name: str,
        approve: bool = False
    ):
        try:
            cli_delete_input = CLIDeleteInput(
                cluster_name = cluster_name,
                approve = approve
            )
            delete_input = DeleteInput(**cli_delete_input.dict())
            Delete(delete_input).run()

        except ValidationError as e:
            cprint("[ERROR] Input value is not correct.", "red")
            validation_result = json.loads(e.json())
            cprint(ValidationFormatter.format(validation_result), "red")
            sys.exit(1)
        except CheckDeniedException as e:
            cprint(e, "red")
            sys.exit(1)
        except TerraformException as e:
            cprint(e, "red")
            cprint("[ERROR] Terraform execution failed.", "red")
            sys.exit(1)



from pathlib import Path
import shutil
from typing import Dict, List

import ruamel.yaml
from termcolor import cprint

from kubetwo.config import ArtifactSettings, Settings
from kubetwo.common import Common
from kubetwo.exception import *


class Ansible:

    def __init__(self, 
        cluster_name: str,
        user_name: str
    ):
        self.cluster_name = cluster_name
        self.user_name = user_name
        self.ssh_private_key_path = Common.get_ssh_private_key_path()
        self.artifact = ArtifactSettings(cluster_name)
    
    def run_kubespray_create(self):
        command = f"""
            ansible-playbook
            --inventory {str(self.artifact.RENDERED_INVENTORY_PATH)}
            --user {self.user_name}
            --private-key {str(self.ssh_private_key_path)}
            --become
            --become-user=root
            cluster.yml
        """
        command = self._format_command(command)
        try:
            cprint("Creating Kubernetes cluster...")
            Common.run_command(command, cwd=str(self.artifact.KUBESPRAY_DIR_PATH))
            cprint("Finished to create Kubernetes cluster.\n")
        except ProcessException as e:
            raise AnsibleKubesprayException(str(e))

    def run_kubespray_scale_in(self, remove_nodes: List[str]):
        remove_nodes = ",".join(remove_nodes)
        command = f"""
            ansible-playbook
            --inventory {str(self.artifact.RENDERED_INVENTORY_PATH)}
            --user {self.user_name}
            --private-key {str(self.ssh_private_key_path)}
            --become
            --become-user=root
            --extra-vars "node={remove_nodes}" 
            --extra-vars "delete_nodes_confirmation=yes"
            remove-node.yml
        """
        command = self._format_command(command)
        try:
            cprint("Scaling in Kubernetes cluster...")
            Common.run_command(command, cwd=str(self.artifact.KUBESPRAY_DIR_PATH))
            cprint("Finished to scale in Kubernetes cluster.\n")
        except ProcessException as e:
            raise AnsibleKubesprayException(str(e))

    def run_kubespray_scale_out(self):
        command = f"""
            ansible-playbook
            --inventory {str(self.artifact.RENDERED_INVENTORY_PATH)}
            --user {self.user_name}
            --private-key {str(self.ssh_private_key_path)}
            --become
            --become-user=root
            scale.yml
        """
        command = self._format_command(command)
        try:
            cprint("Scaling out Kubernetes cluster...")
            Common.run_command(command, cwd=str(self.artifact.KUBESPRAY_DIR_PATH))
            cprint("Finished to scale out Kubernetes cluster.\n")
        except ProcessException as e:
            raise AnsibleKubesprayException(str(e))

    def run_setup(self):
        playbook_dir_path = Settings.ANSIBLE_TEMPLATE_DIR_PATH
        command = f"""
            ansible-playbook
            --inventory {str(self.artifact.RENDERED_INVENTORY_PATH)}
            --user {self.user_name}
            --private-key {str(self.ssh_private_key_path)}
            {Settings.SETUP_PLAYBOOK_NAME}
        """
        command = self._format_command(command)
        try:
            cprint("Setting up Kubernetes cluster...")
            Common.run_command(command, cwd=str(playbook_dir_path))
            cprint("Finished to set up Kubernetes cluster.\n")
        except ProcessException as e:
            raise AnsibleSetupException(str(e))

    def run_fetch_admin_conf(
        self, 
        artifact_dir_path: Path,
        admin_conf_path: Path,
        first_control_plane: Dict
    ):
        playbook_dir_path = Settings.ANSIBLE_TEMPLATE_DIR_PATH
        command = f"""
            ansible-playbook
            --inventory {str(self.artifact.RENDERED_INVENTORY_PATH)}
            --user {self.user_name}
            --private-key {str(self.ssh_private_key_path)}
            --extra-vars dest_path={str(artifact_dir_path)}
            {Settings.FETCH_ADMIN_CONF_NAME}
        """
        command = self._format_command(command)
        try:
            cprint("Fetching admin.conf from control plane...")
            Common.run_command(command, cwd=str(playbook_dir_path))
            self._render_admin_conf(
                artifact_dir_path,
                admin_conf_path,
                first_control_plane,
            )
            cprint("Finished to fetch admin.conf from control plane.\n")
        except ProcessException as e:
            raise AnsibleFetchAdminConfException(str(e))

    def create_inventory_file(self):
        cprint("Rendering inventory file for Ansible...")
        yaml = ruamel.yaml.YAML()
        control_plane_list, worker_node_list = Common.get_node_info(self.artifact.RENDERED_TFSTATE_PATH)

        if len(control_plane_list) % 2 == 0:
            etcd_count = len(control_plane_list) - 1
        else:
            etcd_count = len(control_plane_list)

        with open(Settings.INVENTORY_FILE_PATH, "r") as stream:
            inventory = yaml.load(stream)

        for node in control_plane_list + worker_node_list:
            inventory["all"]["hosts"][node["name"]] = {
                "ansible_ssh_host": node["public_ip"],
                "ip": node["private_ip"],
                "access_ip": node["private_ip"]
            }

        for control_plane in control_plane_list:
            inventory["all"]["children"]["kube_control_plane"]["hosts"][control_plane["name"]] = None

        for worker_node in worker_node_list:
            inventory["all"]["children"]["kube_node"]["hosts"][worker_node["name"]] = None

        for control_plane in control_plane_list[:etcd_count]:
            inventory["all"]["children"]["etcd"]["hosts"][control_plane["name"]] = None

        with open(self.artifact.RENDERED_INVENTORY_PATH, "w") as stream:
            yaml.dump(inventory, stream=stream)

        cprint(f"Finished to render inventory file ({self.artifact.RENDERED_INVENTORY_PATH}).\n")

    def copy_group_vars(self):
        yaml = ruamel.yaml.YAML()
        src = str(self.artifact.KUBESPRAY_GROUP_VARS_DIR_PATH)
        dest = str(self.artifact.RENDERED_KUBESPRAY_GROUP_VARS_DIR_PATH)
        shutil.copytree(src, dest)

        for path in self.artifact.RENDERED_ANSIBLE_DIR_PATH.glob('**/*.yml'):
            if path.name == "inventory.yml":
                continue
            with open(path, "r") as stream:
                group_vars = yaml.load(stream)
            
            if path.name == "k8s-cluster.yml":
                group_vars["cluster_name"] = self.cluster_name
            
            if path.name == "addons.yml":
                group_vars["helm_enabled"] = True

            with open(path, "w") as stream:
                yaml.dump(group_vars, stream=stream)

    def _render_admin_conf(
        self, 
        artifact_dir_path: Path,
        admin_conf_path: Path,
        first_control_plane: Dict
    ):
        yaml = ruamel.yaml.YAML()
        admin_conf_src_path = artifact_dir_path / f"{first_control_plane['name']}/admin.conf"
        with open(admin_conf_src_path, "r") as stream:
            admin_conf = yaml.load(stream)

        clusters = []
        for cluster in admin_conf["clusters"]:
            cluster_info = cluster["cluster"]
            cluster_info.pop("certificate-authority-data", None)
            cluster_info["insecure-skip-tls-verify"] = True
            cluster_info["server"] = f"https://{first_control_plane['public_ip']}:6443"
            cluster["cluster"] = cluster_info
            clusters.append(cluster)
        admin_conf["clusters"] = clusters

        with open(admin_conf_path, "w") as stream:
            yaml.dump(admin_conf, stream=stream)

        shutil.rmtree(str(artifact_dir_path / f"{first_control_plane['name']}"))

    def _format_command(self, command: str) -> str:
        command = command.replace("\n", " ")
        command = " ".join(command.split())
        return command



import json
import os
from pathlib import Path
from typing import Dict, List

from pydantic import BaseModel, conint, constr, root_validator, validator

from kubetwo.config import ArtifactSettings, Settings
from kubetwo.validator import Validator


class CLICreateInput(BaseModel):

    cluster_name: constr(min_length=1, max_length=64)
    ami: str = Settings.DEFAULT_AMI
    instance_type: str = Settings.DEFAULT_INSTANCE_TYPE
    availability_zone: str = None
    control_plane: conint(ge=1) = Settings.DEFAULT_CONTROL_PLANE_NUM
    worker_node: conint(ge=1) = Settings.DEFAULT_WORKER_NODE_NUM
    open_ports: List[conint(ge=-1, le=65535)] = Settings.DEFAULT_OPEN_PORTS
    approve: bool = False

    _is_credentials_registered = root_validator(pre=True, allow_reuse=True)(Validator.is_credentials_registered)

    _is_ssh_keys_registered = root_validator(pre=True, allow_reuse=True)(Validator.is_ssh_keys_registered)

    _is_ssh_keys_exist = root_validator(pre=True, allow_reuse=True)(Validator.is_ssh_keys_exist)

    @validator("availability_zone")
    def set_default_availability_zone(cls, v):
        aws_default_region = os.environ.get("AWS_DEFAULT_REGION")
        if aws_default_region and not v:
            v = aws_default_region + "a"
        return v

    @validator("open_ports")
    def prohibit_port_0(cls, v):
        if 0 in v:
            raise ValueError("Port 0 is not allowed")
        return v


class CLIScaleInput(BaseModel):
    
    cluster_name: constr(min_length=1, max_length=64)
    worker_node: conint(ge=1)
    approve: bool = False

    _is_credentials_registered = root_validator(pre=True, allow_reuse=True)(Validator.is_credentials_registered)

    _is_ssh_keys_registered = root_validator(pre=True, allow_reuse=True)(Validator.is_ssh_keys_registered)

    _is_ssh_keys_exist = root_validator(pre=True, allow_reuse=True)(Validator.is_ssh_keys_exist)

    _artifact_dir_exists = root_validator(pre=True, allow_reuse=True)(Validator.artifact_dir_exists)

    _tfstate_exists = root_validator(pre=True, allow_reuse=True)(Validator.tfstate_exists)

    _input_json_exists = root_validator(pre=True, allow_reuse=True)(Validator.input_json_exists)

    _is_input_json_valid = root_validator(allow_reuse=True)(Validator.is_input_json_valid)

    @validator("worker_node")
    def is_worker_node_changed(cls, v, values):
        with open(ArtifactSettings(values["cluster_name"]).INPUT_JSON_FILE_PATH, "r") as f:
            current_worker_node = json.load(f).get("worker_node")
        if v == current_worker_node:
            raise ValueError(f"Number of worker node should be changed (current value is {v}).")
        return v

    def scale_type(self) -> str:
        with open(ArtifactSettings(self.cluster_name).INPUT_JSON_FILE_PATH, "r") as f:
            input_json = json.load(f)
        return "out" if self.worker_node > input_json["worker_node"] else "in"

    def dict_all(self) -> Dict:
        with open(ArtifactSettings(self.cluster_name).INPUT_JSON_FILE_PATH, "r") as f:
            input_json = json.load(f)
        input_json.update(self.dict())
        input_json["scale_type"] = self.scale_type()
        return input_json


class CLIDeleteInput(BaseModel):

    cluster_name: constr(min_length=1, max_length=64)
    approve: bool = False

    _is_credentials_registered = root_validator(pre=True, allow_reuse=True)(Validator.is_credentials_registered)

    _is_ssh_keys_registered = root_validator(pre=True, allow_reuse=True)(Validator.is_ssh_keys_registered)

    _is_ssh_keys_exist = root_validator(pre=True, allow_reuse=True)(Validator.is_ssh_keys_exist)

    _artifact_dir_exists = root_validator(pre=True, allow_reuse=True)(Validator.artifact_dir_exists)

    _tfstate_exists = root_validator(pre=True, allow_reuse=True)(Validator.tfstate_exists)


class CreateInput(BaseModel):

    cluster_name: str
    ami: str
    instance_type: str
    availability_zone: str
    control_plane: int
    worker_node: int
    open_ports: List[int]
    approve: bool


class ScaleInput(BaseModel):
    
    cluster_name: str
    ami: str
    instance_type: str
    availability_zone: str
    control_plane: int
    worker_node: int
    open_ports: List[int]
    approve: bool


class DeleteInput(BaseModel):

    cluster_name: str
    approve: bool



import json
import os
import re
import sys
from codecs import open
from pathlib import Path
import subprocess
from typing import List

from setuptools import setup
from setuptools.command.install import install


class VersionValidation(install):

    def run(self):
        with open("setup_info.json", "r") as f:
            version = json.load(f)["version"]

        result = subprocess.check_output("git tag -l", shell=True).decode('utf-8')
        registered_versions = [tag.strip("v") for tag in result.split("\n") if tag]

        if not re.match(r"^\d+\.\d+\.\d+$", version):
            print(f"Specified version {version} is invalid format.")
            sys.exit(1)

        if version in registered_versions:
            print(f"Specified version {version} is already registered.")
            sys.exit(1)

        print("Specified version is valid.")


def package_files(directory: str, excelude_suffix_list: List=[]) -> List[str]:
    directory_path = Path("kubetwo") / directory
    paths = []
    excelude_suffix_list = ["." + excelude_suffix for excelude_suffix in excelude_suffix_list]
    for path in directory_path.glob("**/*"):
        if path.suffix in excelude_suffix_list:
            continue
        paths.append(str(path.relative_to("kubetwo")))
    return paths


all_package_files = \
    package_files("ansible_template") + \
    package_files("data") + \
    package_files("terraform_template")

here = Path(os.path.dirname(__file__))

with open(here / "README.md", encoding="utf-8") as f:
    long_description = f.read()

with open(here / "requirements.txt") as f:
    install_requires = f.read().splitlines()

with open("setup_info.json", "r") as f:
    version = json.load(f)["version"]

setup(
    name="kubetwo",
    version=version,
    description="Simple CLI tool to create Kubernetes cluster on AWS EC2.",
    long_description=long_description,
    author="opeco17",
    url="https://github.com/opeco17/kubetwo",
    long_description_content_type="text/markdown",
    packages=["kubetwo"],
    package_data={"kubetwo": all_package_files},
    install_requires=install_requires,
    license="Apache License 2.0",
    python_requires=">= 3.6",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology"
    ],
    keywords="kubetwo kube-two kube2 kube-2",
    entry_points={
        "console_scripts": [
            "kubetwo = kubetwo.cli:main"
        ]
    },
    cmdclass={
        'verify': VersionValidation
    }
)
